#!/usr/bin/env python
# At a random interval between MIN_DELAY_MINS (default: 45 minutes) and 
# MAX_DELAY_MINS (default: 120 minutes) minutes choose an image from 
# IMAGE_DIRECTORY and set it as the desktop wallpaper.

import os
import sys
import time
import random
import signal
import subprocess
from os import path


MIN_DELAY_SECS = int(os.environ.get('MIN_DELAY_MINS', 45)) * 60
MAX_DELAY_SECS = int(os.environ.get('MAX_DELAY_MINS', 120)) * 60
IMAGES_DIRECTORY = path.expanduser(
		os.environ.get('IMAGES_DIRECTORY', '~/.backgrounds'))


class InterruptSignal(KeyboardInterrupt):
	"""Raised from a signal handler to interrupt a system call"""

	def __init__(self, signum):
		KeyboardInterrupt.__init__(self, signum)
		self.signum = signum

	@classmethod
	def sighandler(cls, signum, _):
		raise cls(signum)


def all_files():
	for dirpath, _, filenames in os.walk(IMAGES_DIRECTORY):
		for filename in filenames:
			yield path.join(dirpath, filename)


try:
	signal.signal(signal.SIGUSR1, InterruptSignal.sighandler)
except ValueError:
	# Running on Windows? Looks like it.
	raise OSError("This OS is not supported")

if len(sys.argv) > 0:
	cmd = sys.argv[1:] + [None]
else:
	cmd = ['feh', '--no-fehbg', '--no-xinerama', '--bg-fill', None]

while 1:
	try:
		files = list(all_files())
		if files:
			cmd[-1] = random.choice(files)
			subprocess.call(cmd)
		sleep_time = random.randint(MIN_DELAY_SECS, MAX_DELAY_SECS)
		time.sleep(sleep_time)
	except InterruptSignal:
		pass
