#!/usr/bin/env bash
set -eu
shopt -s globstar

if [ -z "${DEBUG-}" ]; then
	silent_flag=-s
	quit_flag="-c :q"
else
	set -x
fi

unset NVIM_LISTEN_ADDRESS
: ${VIM_BIN:=~/.vim/bin/vim.remote}

SPELL_DIR=`dirname $0`
GIT_DIR=`git rev-parse --git-dir`
declare -a FILES=()

# remove commented-out entries
sed -i.bac '/^#[^#].*/d' "$SPELL_DIR"/**/*.add

for file in "$SPELL_DIR"/**/*.add; do
	mv "${file}" "${file}.tmp"
	sort -f -u "${file}.tmp" > "${file}"
	rm "${file}.tmp"
	${VIM_BIN} -E ${silent_flag-} -c ':mkspell! %' ${quit_flag-} "${file}"
	[[ $(basename $file) =~ ^en ]] || FILES+=( "${file}" "${file}.spl" )
done

[ ${#FILES[@]} -gt 0 ] || exit 0

if [[ -v NO_COMMIT ]]; then
	echo >&2 "Not committing changes"
elif [ -d "$GIT_DIR"/rebase-merge ]; then
	git add "${FILES[@]}"
	git rebase --continue
elif [ -e "$GIT_DIR"/MERGE_MSG ]; then
	git commit --include "${FILES[@]}"
else
	git add "${FILES[@]}"
	git commit -F- <<-END_MESSAGE
		Add vim spell file updates from `hostname` `date +%D`

		Automatically generated:
		  source: .vim/spell/update.sh
		  time:   `date`
		  host:   `hostname -f`
	END_MESSAGE
fi
